@@ -1,5 +1,6 @@
 
 
+
 #' @title Adds a menu based GUI for updating R within Rgui
 #' @description Adds a menu based GUI for updating R within Rgui.
 #' @details
@@ -14,8 +15,8 @@
 #' add.installr.GUI() 
 #' }
 add.installr.GUI <- function() {
-   # Thanks to Dason: http://stackoverflow.com/questions/15250487/how-to-add-a-menu-item-to-rgui/15250992?iemail=1#15250992
-   # Add GUI (only in Windows's Rgui)
+   require(utils) # needed for winMenuNames etc.
+   
    if(is.windows() & is.Rgui() & !is.RStudio()){
       Update_in_winMenuNames <- "Update" %in% winMenuNames() # I'm making sure this function wasn't used before.  If it was, then running it again might cause bugs...   
       if(!Update_in_winMenuNames) {
@@ -24,6 +25,17 @@ add.installr.GUI <- function() {
          winMenuAddItem("Update", "Update R packages", "update.packages(ask = F)")      
          winMenuAddItem("Update", "Install software", "installr()")
          winMenuAddItem("Update", "Manage Windows", "os.manage()")      
+         
+         
+         # add a menu for adding/removing the installr package to startup
+         # based on whether or not it is already setup to run on startup.
+         if(is_in_.First_in_Rprofile.site("require(installr)")) {
+            add_remove_installr_from_startup_menu()
+         } else {
+            add_load_installr_on_startup_menu()   
+         }        
+         
+         
          return(invisible(TRUE))         
       } else {
          warning("Can not add a new menu item for installr since the menu already has 'Update' in it")   
@@ -35,6 +47,11 @@ add.installr.GUI <- function() {
 }
 
 
+
+
+
+
+
 #' @title Removes the menu based GUI for updating R within Rgui
 #' @description Removes the menu based GUI for updating R within Rgui.
 #' @details
@@ -64,7 +81,7 @@ remove.installr.GUI <- function() {
    # Thanks for Romain: http://stackoverflow.com/questions/4369334/first-lib-idiom-in-r-packages
    
    # adding and removing menus from the Rgui when loading and detaching the library
-   setHook(packageEvent("installr", "attach"), {function(pkgname, libpath) {add.installr.GUI()}  } )
+   setHook(packageEvent("installr", "attach"), {function(pkgname, libpath) {tryCatch(add.installr.GUI(), error = function(e) invisible(FALSE))}  } )
    setHook(packageEvent("installr", "detach"), {function(pkgname, libpath) {remove.installr.GUI()}  } )
    
 }
@@ -86,19 +103,19 @@ remove.installr.GUI <- function() {
 installrWelcomeMessage <- function(){
    
    paste("\n",     
-         "Welcome to installr version ", utils:::packageDescription("installr")$Version, "\n",
+         "Welcome to installr version ", utils::packageDescription("installr")$Version, "\n",
          "\n",
          # "Type ?installr to access the overall documentation and\n",
          # "vignette('installr') for the package vignette.\n",
          # "You can execute a demo of the package via: demo(installr)\n",
          # "\n",  
-         "More information is available on the installr project web-site:\n",
+         "More information is available on the installr project website:\n",
          "https://github.com/talgalili/installr/\n",
          "\n",               
          "Contact: <tal.galili@gmail.com>\n",
          "Suggestions and bug-reports can be submitted at: https://github.com/talgalili/installr/issues\n",
          "\n",
-         "\t\t\tTo suppress the this message use:\n",
+         "\t\t\tTo suppress this message use:\n",
          "\t\t\tsuppressPackageStartupMessages(library(installr))\n",  
          sep="")
 }
@@ -135,5 +152,9 @@ installrWelcomeMessage <- function(){
 # when a function is renamed, its document in man must be removed - otherwise it may cause problems with the built check (it will try to run the code in the example, and will fail.)
 # When all is done, run:
 # require(devtools)
+# check()
+# check(args="--as-cran")
+#                 Thanks to: http://stackoverflow.com/questions/10017702/r-cmd-check-options-for-more-rigorous-testing-2-15-0
+# file.copy("NEWS", "NEWS.md")
 # build_win()
-# release()
+# release(check = TRUE)
